"""ASGI-Tools Utils."""

from functools import wraps
from inspect import isasyncgenfunction, iscoroutinefunction
from typing import Awaitable, Callable

from multidict import CIMultiDict

from asgi_tools.typing import ScopeHeaders


def is_awaitable(fn: Callable) -> bool:
    """Check than the given function is awaitable."""
    return iscoroutinefunction(fn) or isasyncgenfunction(fn)


def to_awaitable(fn: Callable) -> Callable[..., Awaitable]:
    """Convert the given function to a coroutine function if it isn't"""
    if is_awaitable(fn):
        return fn

    @wraps(fn)
    async def coro(*args, **kwargs):
        return fn(*args, **kwargs)

    return coro


def parse_headers(headers: ScopeHeaders) -> CIMultiDict:
    """Decode the given headers list."""
    return CIMultiDict([(n.decode("latin-1"), v.decode("latin-1")) for n, v in headers])



"""ASGI Types."""

from typing import Any, Awaitable, Callable, Dict, List, MutableMapping, Tuple, TypeVar, Union

Message = MutableMapping[str, Any]
Send = Callable[[Message], Awaitable[Any]]
Receive = Callable[[], Awaitable[Message]]
Scope = MutableMapping[str, Any]
ScopeHeaders = List[Tuple[bytes, bytes]]
JSONType = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]
ASGIApp = Callable[[Scope, Receive, Send], Awaitable]

ResponseContent = Union[str, bytes]
F = TypeVar("F", bound=Callable)
DecoratedCallable = TypeVar("DecoratedCallable", bound=Callable)



"""Setup the package."""


# Parse requirements
# ------------------
import pkg_resources
import pathlib


def parse_requirements(path: str) -> 'list[str]':
    with pathlib.Path(path).open() as requirements:
        return [str(req) for req in pkg_resources.parse_requirements(requirements)]


# Setup extensions
# ----------------
import os
import sys
from setuptools import Extension

NO_EXTENSIONS = (
    sys.implementation.name != 'cpython' or
    bool(os.environ.get("ASGI_TOOLS_NO_EXTENSIONS"))
)
EXT_MODULES = [] if NO_EXTENSIONS else [
    Extension("asgi_tools.multipart", ["asgi_tools/multipart.c"], extra_compile_args=['-O2']),
    Extension("asgi_tools.forms", ["asgi_tools/forms.c"], extra_compile_args=['-O2']),
]


# Setup package
# -------------
from setuptools import setup

setup(
    setup_requires=["wheel"],

    ext_modules=EXT_MODULES,

    install_requires=parse_requirements('requirements/requirements.txt'),
    extras_require={
        'tests': parse_requirements('requirements/requirements-tests.txt'),
        'build': parse_requirements('requirements/requirements-build.txt'),
        'docs': parse_requirements('requirements/requirements-docs.txt'),
        'examples': parse_requirements('requirements/requirements-examples.txt'),
        'orjson': parse_requirements('requirements/requirements-orjson.txt'),
        'ujson': parse_requirements('requirements/requirements-ujson.txt'),
    },
)

# pylama:ignore=E402,D
